@@ -698,6 +698,7 @@ command_line_proxy_ready (GDBusConnection *connection,
 
 	g_dbus_connection_send_message_with_reply (g_dbus_proxy_get_connection (proxy),
 	                                           message,
+	                                           G_DBUS_SEND_MESSAGE_FLAGS_NONE,
 	                                           -1,
 	                                           NULL,
 	                                           NULL,
@@ -712,7 +713,7 @@ handle_slave (GeditDBus *dbus)
 {
 	/* send the requested commands to the master */
 	GDBusConnection *conn;
-	GDBusCallFlags flags;
+	GDBusProxyFlags flags;
 	GeditCommandLine *command_line;
 
 	conn = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
@@ -732,7 +733,7 @@ handle_slave (GeditDBus *dbus)
 	}
 
 	g_dbus_proxy_new (conn,
-	                  G_BUS_NAME_WATCHER_FLAGS_NONE,
+	                  flags,
 	                  NULL,
 	                  "org.gnome.gedit",
 	                  "/org/gnome/gedit",
@@ -826,7 +827,7 @@ window_from_display_arguments (gboolean           new_window,
                                DisplayParameters *dparams,
                                gboolean           create)
 {
-	GdkScreen *screen;
+	GdkScreen *screen = NULL;
 	GeditApp *app;
 	GeditWindow *ret;
 
